use crate::{solve1, solve2};
static INPUT: &str = include_str!("../../data/year2022/day04.txt");
static SAMPLE_INPUT: &str = include_str!("../../data/year2022/day04_test.txt");

struct AssignmentPair {
    l1: i64,
    l2: i64,
    r1: i64,
    r2: i64,
}

impl AssignmentPair {
    fn fully_contained(self) -> Option<()> {
        if (self.l1 <= self.r1 && self.l2 >= self.r2) || (self.r1 <= self.l1 && self.r2 >= self.l2)
        {
            return Some(());
        }
        None
    }

    fn overlap(self) -> Option<()> {
        if self.l1 >= self.r1 && self.l1 <= self.r2
            || self.l2 >= self.r1 && self.l2 <= self.r2
            || self.r1 >= self.l1 && self.r1 <= self.l2
            || self.r2 >= self.l1 && self.r2 <= self.l2
        {
            return Some(());
        }
        None
    }
}

fn parse_input(line: &str) -> AssignmentPair {
    let mut split = line.split(',');
    let (left, right) = (
        split.next().expect("No left side"),
        split.next().expect("No right side"),
    );
    let mut left_split = left.split('-');
    let mut right_split = right.split('-');
    let (l1, l2) = (
        left_split.next().expect("No left side"),
        left_split.next().expect("No right side"),
    );
    let (r1, r2) = (
        right_split.next().expect("No left side"),
        right_split.next().expect("No right side"),
    );
    AssignmentPair {
        l1: l1.parse().expect("Not a number"),
        l2: l2.parse().expect("Not a number"),
        r1: r1.parse().expect("Not a number"),
        r2: r2.parse().expect("Not a number"),
    }
}

fn solve1(input: &str) -> i64 {
    input
        .lines()
        .map(parse_input)
        .filter_map(AssignmentPair::fully_contained)
        .count() as i64
}

fn solve2(input: &str) -> i64 {
    input
        .lines()
        .map(parse_input)
        .filter_map(AssignmentPair::overlap)
        .count() as i64
}

solve1!(2, 547);
solve2!(4, 843);
